#ifndef PARSER_H
#define PARSER_H

#include "lexer.h"
#include <stdint.h>

typedef enum {
    ExprTypeError,
    ExprTypeInt,
    ExprTypeUnary,
    ExprTypeBinary,
} ExprType;

typedef struct Expr Expr;

typedef struct {
    Position pos;
    char* message;
} ErrorExpr;

typedef struct {
    int64_t value;
} IntExpr;

typedef enum {
    UnaryExprTypeNegate,
} UnaryExprType;

typedef struct {
    UnaryExprType type;
    Expr* subject;
} UnaryExpr;

typedef enum {
    BinaryExprTypeAdd,
    BinaryExprTypeSubtract,
    BinaryExprTypeMultiply,
    BinaryExprTypeDivide,
} BinaryExprType;

typedef struct {
    BinaryExprType type;
    Expr* left;
    Expr* right;
} BinaryExpr;

struct Expr {
    ExprType type;
    union {
        ErrorExpr error;
        IntExpr int_expr;
        UnaryExpr unary;
        BinaryExpr binary;
    };
};

Expr* parse(Lexer* lexer, const char* text, size_t length);
void free_expr(Expr* expr);
char* expr_to_string(const Expr* expr);

#endif
